import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:wowit/message.dart';
import 'dart:io';
import 'package:http/http.dart' as http;

import 'common.dart';

class Methods {
  static String showDate(DateTime date) {
    return '${date.day}/${date.month}/${date.year}';
  }

  static double round(double d, int r) {
    String s = d.toStringAsFixed(r);
    return double.parse(s);
  }

//do khoan cach thoi gian bien truyen vao so voi hien tai
  static String timeAgoString(DateTime date) {
    var secondsInterval = DateTime.now().difference(date).inSeconds;
    if (secondsInterval < 10) {
      return Message.getMessage('VAI_GIAY_TRUOC');
    }
    if (secondsInterval < 60) {
      return secondsInterval.toString() +
          ' ' +
          Message.getMessage('GIAY_TRUOC');
    }
    double minutes = (secondsInterval / 60.0);
    minutes = round(minutes, 1);
    if (minutes < 60) {
      return minutes.toString() + ' ' + Message.getMessage('PHUT_TRUOC');
    }
    var hours = round((minutes / 60.0), 1);
    if (hours < 24) {
      return hours.toString() + ' ' + Message.getMessage('GIO_TRUOC');
    }
    var days = round((hours / 60.0), 1);
    if (days < 30) {
      return days.toString() + ' ' + Message.getMessage('NGAY_TRUOC');
    }
    var months = round((days / 30.0), 1);
    if (months < 12) {
      return months.toString() + ' ' + Message.getMessage('THANG_TRUOC');
    }
    var years = round((months / 12.0), 1);
    return years.toString() + ' ' + Message.getMessage('NAM_TRUOC');
  }

  static Object getDataHashMap(Map data, String key,
      {String defaultValue = ""}) {
    try {
      dynamic val;
      if (data.containsKey(key)) {
        val = data[key];
        if (val != null && val.toString() != "") {
          return val;
        }
      }
      return defaultValue;
    } catch (e) {
      print('getDataHashMap ex:' + e.toString());
      return defaultValue;
    }
  }

  static Object getDataHashMapList(Map data, String key, int index,
      {String defaultValue = ""}) {
    try {
      var arrKeys = key.split('.');
      if (arrKeys.length != 2) return defaultValue;
      if (data.containsKey(arrKeys[0])) {
        List<dynamic> lstObj = data[arrKeys[0]] as List<dynamic>;
        if (lstObj != null && lstObj.length > index) {
          Map map = lstObj[index];
          return getDataHashMap(map, arrKeys[1], defaultValue: defaultValue);
        }
      }
      return defaultValue;
    } catch (e) {
      print('getDataHashMap ex:' + e.toString());
      return defaultValue;
    }
  }

  static String getURL(String path) {
    return Common.url_host + '/' + path;
  }

  static String getFullURLFileHosting(
      String collection, int documentID, String fileName) {
    String url;
    if (documentID != -1) {
      url = Common.url_host +
          '/images/Upload/' +
          collection +
          '/' +
          documentID.toString() +
          '/' +
          fileName;
    } else {
      url = Common.url_host + '/images/Upload/' + collection + '/' + fileName;
    }

    return url;
  }

  static double stringToDouble(String s) {
    try {
      double d = double.parse(s);
      return d;
    } catch (e) {
      print('stringToDouble:' + e.toString());
      return 0;
    }
  }

  static int stringToInt(String s) {
    try {
      double d = stringToDouble(s);
      return d.floor();
      // int i = int.parse(strs[0]);
      // return i;
    } catch (e) {
      print('stringToInt:' + e.toString());
      return 0;
    }
  }

  static bool stringToBool(String s, {bool defaultValue = false}) {
    try {
      return s == "1" || s.toLowerCase() == "true";
    } catch (e) {
      print('stringToBool:' + e.toString());
      return defaultValue;
    }
  }

  static DateTime stringToDateTime(String s) {
    try {
      var parsedDate = DateTime.parse(s);
      return parsedDate;
    } catch (e) {
      print('stringToTimestamp:' + e.toString());
      return new DateTime(1990);
    }
  }

//doc tien thanh chu
//lienHe : neu so tien = 0 -> tra ve 'Lien He'
  static String docTienThanhChu(double soTien,
      {int roundDecimal = 2, bool lienHe = true}) {
    int i;

    if (soTien > 8999999999999999) {
      return soTien.toString();
    }

    List<String> tienDonVi = [
      Message.getMessage('DON_VI_TIEN_TRIEU_TY'),
      Message.getMessage('DON_VI_TIEN_NGIN_TY'),
      Message.getMessage('DON_VI_TIEN_TY'),
      Message.getMessage('DON_VI_TIEN_TRIEU'),
      Message.getMessage('DON_VI_TIEN_NGIN'),
      Message.getMessage('DON_VI_TIEN_DONG')
    ];

    List<double> tienSo = [
      1000000000000000.0,
      1000000000000.0,
      1000000000.0,
      1000000.0,
      1000.0,
      1
    ];
    for (i = 0; i < tienSo.length; i++) {
      if (soTien / tienSo[i] > double.maxFinite ||
          soTien / tienSo[i] < double.minPositive) {
        return soTien.toString();
      }

      if (soTien >= tienSo[i]) {
        if (soTien % tienSo[i] > 0) {
          if (roundDecimal > 0) {
            var r = round((soTien / tienSo[i]), roundDecimal);

            return r.toString() + " " + tienDonVi[i];
          } else {
            double lastValue = (soTien / tienSo[i]);

            if (lastValue == 0 && lienHe) {
              return Message.getMessage('GIA_LIEN_HE');
            }
            return (lastValue).toString() + " " + (tienDonVi[i]);
          }
        } else {
          return (stringToInt((soTien / tienSo[i]).toString())).toString() +
              " " +
              tienDonVi[i];
        }
      }
    }
    if (soTien == 0 && lienHe) {
      return Message.getMessage('GIA_LIEN_HE');
    }
    return soTien.toString();
  }

  static String getChanelId(int uidFrom, int uidTo) {
    return (uidFrom > uidTo)
        ? (uidFrom.toString() + '_' + uidTo.toString())
        : (uidTo.toString() + '_' + uidFrom.toString());
  }

  static List<String> getDataHashMapListString(Map data, String key,
      {String defaultValue = ""}) {
    List<String> lst = [];
    try {
      var arrKeys = key.split('.');
      if (arrKeys.length != 2) return lst;
      if (data.containsKey(arrKeys[0])) {
        List<dynamic> lstObj = data[arrKeys[0]] as List<dynamic>;
        if (lstObj != null && lstObj.length > 0) {
          for (int index = 0; index < lstObj.length; index++) {
            Map map = lstObj[index];
            lst.add(
                getDataHashMap(map, arrKeys[1], defaultValue: defaultValue));
          }
        }
        return lst;
      }
      return lst;
    } catch (e) {
      return lst;
    }
  }

  static List<Map> getDataHashMapListObject(Map data, String key) {
    List<Map> lst = [];
    try {
      if (data.containsKey(key)) {
        List<dynamic> lstObj = data[key] as List<dynamic>;
        if (lstObj != null && lstObj.length > 0) {
          for (int index = 0; index < lstObj.length; index++) {
            Map map = lstObj[index];
            lst.add(map);
          }
        }
        return lst;
      }
      return lst;
    } catch (e) {
      print('getDataHashMapListObject ex:' + e.toString());
      return lst;
    }
  }

  static DateTime stringToDatime(String dateTime) {
    return DateTime.parse(dateTime);
  }

  static String preparePhone(String phone) {
    String phoneNumber;

    if (phone.startsWith('0')) {
      phoneNumber = "+84" + phone.substring(1);
    } else {
      phoneNumber = phone;
    }
    return phoneNumber;
  }

  static void showWatting() {
    showDialog(
      context: Get.overlayContext,
      barrierDismissible: false,
      builder: (_) => WillPopScope(
        onWillPop: () async => false,
        child: Center(
          child: SizedBox(
            width: 60,
            height: 60,
            child: CircularProgressIndicator(
              strokeWidth: 10,
            ),
          ),
        ),
      ),
    );
  }

  static String getURLUploadImage() {
    return Common.url_host + '/File/UploadImage';
  }

  static Future<String> uploadFile(File file, String collection, int documentID,
      {String fileName = ""}) async {
    //create multipart request for POST or PATCH method
    if (fileName == null || fileName == '') {
      fileName = getFileName(file.path);
    }
    var request = http.MultipartRequest("POST", Uri.parse(getURLUploadImage()));
    //add text fields
    request.fields["collection"] = collection;
    request.fields["documentID"] = documentID.toString();
    request.fields["fileName"] = fileName;
    //create multipart using filepath, string or bytes
    var pic = await http.MultipartFile.fromPath("file", file.path);
    //add multipart to request
    request.files.add(pic);
    var response = await request.send();

    //Get the response from the server
    var responseData = await response.stream.toBytes();
    var responseString = String.fromCharCodes(responseData);
    return responseString;
  }

  static String getFileName(String fullPath) {
    String fileName = fullPath;
    if (fullPath.contains('/')) {
      var arr = fullPath.split('/');
      fileName = arr[arr.length - 1];
    }
    return fileName;
  }

  static Future<void> logout () async
  {
    await FirebaseAuth.instance.signOut();
  }
}
